package formFaces.action;

import javax.servlet.ServletRequest;

import java.util.Date;
import java.util.Enumeration;
import java.util.List;
import java.util.ArrayList;

import formFaces.excepcio.FormFacesExcepcio;

public abstract class ActionForm extends Classe {
	private JspResolver lastResolver;
	private Status lastStatus;
	private Validacio lastValidacioStatus;
	private List<JspResolver> resolver;
	
	public ActionForm(){
		super();
		lastStatus = null;
		lastResolver = null;
		lastValidacioStatus = null;
		resolver = new ArrayList<JspResolver>();
		definicioResolucioJsp();
	}
	

	public void map(ServletRequest request) {
		for (Enumeration<String> par = super.getClauParametres(); par.hasMoreElements();){
			String param = par.nextElement();
			String valor = request.getParameter(param);
			if (valor == null){
				super.novaExcepcio(new FormFacesExcepcio("Parametre passat "+param+" no creat al formulari", 5));
			} else {
				super.setValor(param, valor);
			}
		}
	}
	
	
	@Override
	/**
	 * Definici del formulari
	 * 		addParametre(String, String) crea els parmetres. 
	 * 									El primer String es el nom i el segon es el tipus de parametre que pot ser:
	 * 									-	String
	 * 									- 	Integer
	 * 									-	Float
	 * 									-	Long
	 * 									-	Double
	 * 									-	Date		Els valors de les dates en format AAAAMMDD
	 * 									-	Byte
	 * 									-	Short
	 * 									-	Boolean
	 * 		addValidacio(String) crea una validaci per l'ultim parmetre. String es la comparaci que pot ser:
	 * 									-	le	menor o igual
	 * 									-	lt	menor
	 * 									-	ge	major o igual
	 * 									-	gt	major
	 * 									-	eq	igual
	 * 									-	ne	diferent
	 * 									-	bt	entre dos valors inclosos
	 * 									-	nb	fora de dos valors
	 * 									-	in	est en una llista de valors
	 * 									-	ni	no est en una llista de valors
	 * 		addValor(Comparable) afegeix un valor a comparar a la ltima validaci
	 * 		addValidacio(String, Comparable) crea una validaci i li afegeix un valor a comparar
	 * 		addValidacio(String, Comparable, Comparable) crea una validaci i li afegeix dos valors a comparar
	 * 		addClasse(String, String) crea les classes que porten les dades del formulari
	 * 									el primer String es el nom de la classe
	 * 									el segon String es el tipus de classe
	 * 		addList(String, String) crea les llistes de classes que portaran els valors
	 * 									el primer String es el nom de la llista
	 * 									el segon String es el tipus de classe
	 * 									 
	 */
	protected abstract void definicio();
	
	/**
	 * 		addJsp(String) crea un nou JspResolver per resodre aquest Jsp
	 * 		addJspStatus(String, String) crea un nou Status i l'afegeix al JspResolver. El primer String es el tipus d'estat.
	 * 										Els tipus dpestat possibles son:		
	 * 												. "llista", nomLlista				Ok si la llista te elements
	 * 												- "parametre", nomParametre			Ok si el parametre compleis una validacio
	 * 												- "classe", nomClasse.nomParametre  Ok si el parametre de la classe compliex ina validacio.	
	 * 		addValidacioStatus(String) crea una validaci per l'ultim Status. String es la comparaci que est definida a vaidacio
	 * 		addValorStatus(Comparable) afegeix un valor a comparar a l'ltima validaci
	 * 		addValidacioStatus(String, Comparable) crea una validacio per l'ultim status i li afegeix un valor a comparar
	 * 		addValidacioStatus(String, Comparable, Comparable) crea una validacio per l'ultim status i li afegeix dos valors a comparar
	 * 
	 */
	protected abstract void definicioResolucioJsp();
	
	protected void addJsp(String jsp){
		lastResolver = new JspResolver(jsp);
		resolver.add(lastResolver);
	}

	protected void addJspStatus(String tipus, String camp){
		if (lastResolver != null){
			lastStatus = lastResolver.setEstat(tipus, camp);
		} else {
			excepcions.nova(new FormFacesExcepcio("addJspStatus ha danar desprs d'un addParametre", 5));
		}
	}
	
	protected void addValidacioStatus(String v){
		if (lastStatus != null){
			lastValidacioStatus = new Validacio(v);
			lastStatus.setValidacio(lastValidacioStatus);
		} else {
			excepcions.nova(new FormFacesExcepcio("addValidacioStatus ha danar desprs d'un addParametre", 5));
		}
	}
	
	protected void addValorStatus(String s){
		try {
			if (lastValidacioStatus != null){
				Comparable cmp = null;
				String t = lastStatus.getTipusCamp(this);
				if (t!=null){
					if(t.equalsIgnoreCase("String")){
						cmp = new String(s);
					}
					if(t.equalsIgnoreCase("Integer")){
						cmp = new Integer(s);
					}
					if(t.equalsIgnoreCase("Float")){
						cmp = new Float(s);
					}
					if(t.equalsIgnoreCase("Long")){
						cmp = new Long(s);
					}
					if(t.equalsIgnoreCase("Double")){
						cmp = new Double(s);
					}
					if(t.equalsIgnoreCase("Date")){
						cmp = new Date((new Integer(s.substring(0, 4))).intValue()-1900, (new Integer(s.substring(4, 6))).intValue()-1, (new Integer(s.substring(6, 8))).intValue());
					}
					if(t.equalsIgnoreCase("Boolean")){
						cmp = new Boolean(s);
					}		
					if(cmp != null){
						lastValidacioStatus.addValor(cmp);
					}
				} else {
					excepcions.nova(new FormFacesExcepcio("addValorStatus ha danar desprs d'un addStatus", 5));
				}
			} else {
				excepcions.nova(new FormFacesExcepcio("addValorStatus ha danar desprs d'un addValidacioStatus", 5));
			}
		} catch (Exception e){
			excepcions.nova(new FormFacesExcepcio("El valor "+s+" d'un status no es del tipus correcte", 5));
		}
	}
	
	protected void addValorStatus(int i){
		addValorStatus(new Integer(i).toString());
	}
	
	protected void addValorStatus(float f){
		addValorStatus(new Float(f).toString());
	}

	protected void addValorStatus(long l){
		addValorStatus(new Long(l).toString());
	}
	
	protected void addValorStatus(double d){
		addValorStatus(new Double(d).toString());
	}

	protected void addValidacioStatus(String v, String s){
		addValidacioStatus(v);
		addValorStatus(s);
	}

	protected void addValidacioStatus(String v, int i){
		addValidacioStatus(v);
		addValorStatus(i);
	}

	protected void addValidacioStatus(String v, float f){
		addValidacioStatus(v);
		addValorStatus(f);
	}

	protected void addValidacioStatus(String v, long l){
		addValidacioStatus(v);
		addValorStatus(l);
	}

	protected void addValidacioStatus(String v, double d){
		addValidacioStatus(v);
		addValorStatus(d);
	}

	protected void addValidacioStatus(String v, String s1, String s2){
		addValidacioStatus(v);
		addValorStatus(s1);
		addValorStatus(s2);
	}

	protected void addValidacioStatus(String v, int i1, int i2){
		addValidacioStatus(v);
		addValorStatus(i1);
		addValorStatus(i2);
	}

	protected void addValidacioStatus(String v, float f1, float f2){
		addValidacioStatus(v);
		addValorStatus(f1);
		addValorStatus(f2);
	}

	protected void addValidacioStatus(String v, long l1, long l2){
		addValidacioStatus(v);
		addValorStatus(l1);
		addValorStatus(l2);
	}

	protected void addValidacioStatus(String v, double d1, double d2){
		addValidacioStatus(v);
		addValorStatus(d1);
		addValorStatus(d2);
	}
	
	public String resolJsp(){
		String jsp = null;
		int i = 0;
		while (jsp==null & i < resolver.size()){
			jsp = resolver.get(i).resolve(this);
//			System.out.println("ressoluci: "+i+" - "+jsp);
			i++;
		}
		return jsp;
	}

}
