package formFaces.action;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;
import java.util.Date;


import formFaces.excepcio.*;


public abstract class Classe {
	
	private Hashtable<String, Parametre > parametres;
	private Hashtable<String, Classe > classes;
	private Hashtable<String, List<Classe> > llistes;
	private Hashtable<String, String> classeLlista;
	private Parametre lastParametre;
	private Validacio lastValidacio;
	protected Excepcions excepcions;
	
	public Classe() {
		parametres 		= new Hashtable<String, Parametre>();
		classes 		= new Hashtable<String, Classe>();
		llistes			= new Hashtable<String, List<Classe>>();
		classeLlista 	= new Hashtable<String, String>(); 
		lastParametre 	= null;
		lastValidacio 	= null;
		excepcions 		= new Excepcions();
		definicio();
	}
	
	/**
	 * Definici del formulari
	 * 		addParametre(String, String) crea els parmetres. 
	 * 									El primer String es el nom i el segon es el tipus de parametre que pot ser:
	 * 									-	String
	 * 									- 	Integer
	 * 									-	Float
	 * 									-	Long
	 * 									-	Double
	 * 									-	Date		Els valors de les dates en format AAAAMMDD
	 * 									-	Byte
	 * 									-	Short
	 * 									-	Boolean
	 * 		addValidacio(String) crea una validaci per l'ultim parmetre. String es la comparaci que pot ser:
	 * 									-	le	menor o igual
	 * 									-	lt	menor
	 * 									-	ge	major o igual
	 * 									-	gt	major
	 * 									-	eq	igual
	 * 									-	ne	diferent
	 * 									-	bt	entre dos valors inclosos
	 * 									-	nb	fora de dos valors
	 * 									-	in	est en una llista de valors
	 * 									-	ni	no est en una llista de valors
	 * 		addValor(Comparable) afegeix un valor a comparar a la ltima validaci
	 * 		addValidacio(String, Comparable) crea una validaci i li afegeix un valor a comparar
	 * 		addValidacio(String, Comparable, Comparable) crea una validaci i li afegeix dos valors a comparar
	 * 		addClasse(String, String) crea les classes que porten les dades del formulari
	 * 									el primer String es el nom de la classe
	 * 									el segon String es el tipus de classe
	 * 		addList(String, String) crea les llistes de classes que portaran els valors
	 * 									el primer String es el nom de la llista
	 * 									el segon String es el tipus de classe
	 * 									 
	 */
	protected abstract void definicio();
	
	protected void addParametre(String s, String t){
		lastParametre = new Parametre(t);
		parametres.put(s, lastParametre);
	}
	
	protected void addValidacio(String v){
		if (lastParametre != null){
			lastValidacio = new Validacio(v);
			lastParametre.setValidacio(lastValidacio);
		} else {
			excepcions.nova(new FormFacesExcepcio("addValidacio ha danar desprs d'un addParametre", 5));
		}
	}
	
	protected void addValor(String s){
		try {
			if (lastValidacio != null){
				Comparable cmp = null;
				String t = lastParametre.getTipus();
				if(t.equalsIgnoreCase("String")){
					cmp = new String(s);
				}
				if(t.equalsIgnoreCase("Integer")){
					cmp = new Integer(s);
				}
				if(t.equalsIgnoreCase("Float")){
					cmp = new Float(s);
				}
				if(t.equalsIgnoreCase("Long")){
					cmp = new Long(s);
				}
				if(t.equalsIgnoreCase("Double")){
					cmp = new Double(s);
				}
				if(t.equalsIgnoreCase("Date")){
					cmp = new Date((new Integer(s.substring(0, 4))).intValue()-1900, (new Integer(s.substring(4, 6))).intValue()-1, (new Integer(s.substring(6, 8))).intValue());
				}
				if(t.equalsIgnoreCase("Boolean")){
					cmp = new Boolean(s);
				}		
				if(cmp != null){
					lastValidacio.addValor(cmp);
				}
			} else {
				excepcions.nova(new FormFacesExcepcio("addValor ha danar desprs d'un addValidacio", 5));
			}
		} catch (Exception e){
			excepcions.nova(new FormFacesExcepcio("El valor "+s+" d'una validacio no es del tipus correcte", 5));
		}
	}
	
	protected void addValor(int i){
		addValor(new Integer(i).toString());
	}
	
	protected void addValor(float f){
		addValor(new Float(f).toString());
	}

	protected void addValor(long l){
		addValor(new Long(l).toString());
	}
	
	protected void addValor(double d){
		addValor(new Double(d).toString());
	}

	protected void addValidacio(String v, String s){
		addValidacio(v);
		addValor(s);
	}

	protected void addValidacio(String v, int i){
		addValidacio(v);
		addValor(i);
	}

	protected void addValidacio(String v, float f){
		addValidacio(v);
		addValor(f);
	}

	protected void addValidacio(String v, long l){
		addValidacio(v);
		addValor(l);
	}

	protected void addValidacio(String v, double d){
		addValidacio(v);
		addValor(d);
	}

	protected void addValidacio(String v, String s1, String s2){
		addValidacio(v);
		addValor(s1);
		addValor(s2);
	}

	protected void addValidacio(String v, int i1, int i2){
		addValidacio(v);
		addValor(i1);
		addValor(i2);
	}

	protected void addValidacio(String v, float f1, float f2){
		addValidacio(v);
		addValor(f1);
		addValor(f2);
	}

	protected void addValidacio(String v, long l1, long l2){
		addValidacio(v);
		addValor(l1);
		addValor(l2);
	}

	protected void addValidacio(String v, double d1, double d2){
		addValidacio(v);
		addValor(d1);
		addValor(d2);
	}
	

	protected void addClasse(String n, String nc){
		try{
			Class c = Class.forName("form_faces.classes."+nc);
			Classe cl = (Classe)c.newInstance();
			classes.put(n, cl);
		} catch (Exception e){
			  excepcions.nova(new FormFacesExcepcio("Error al cear la Classe "+ nc, 5));
		}
	}
	
	protected void addList(String n, String nc){
		try{
			List<Classe> lcl = new ArrayList<Classe>();
			llistes.put(n, lcl);
			classeLlista.put(n, "form_faces.classes."+nc);
		} catch (Exception e){
			  excepcions.nova(new FormFacesExcepcio("Error al cear la Classe "+ nc, 5));
		}
	}
	
	public void setValor(String param, String valor){
		if (parametres.containsKey(param)){
			Parametre p = parametres.get(param);
			try {
				p.setValor(valor);
			} catch (FormFacesExcepcio e){
				excepcions.nova(new FormFacesExcepcio("Parmetre "+param+" "+e.getMessage(),e.getGravetat()));
			}
		} else {
			  excepcions.nova(new FormFacesExcepcio("Error parmetre inexistent "+ param, 4));
		}
	}
	
	public String getTipusParametre(String param){
		if (parametres.containsKey(param)){
			return parametres.get(param).getTipus();
		} else {
			return null;
		}
	}
	
	public String getValor(String param){
		if (parametres.containsKey(param)){
			if (parametres.get(param)==null){
				return null;
			} else {
				return parametres.get(param).toString();
			}
		} else {
			return null;
		}
	}
	
	public Comparable getParametre(String param){
		if (parametres.containsKey(param)){
			return parametres.get(param).getParametre();
		} else {
			return null;
		}
	}
	
	public Enumeration<String> getClauParametres(){
		return parametres.keys();
	}

	public Excepcions getExcepcions(){
		return getExcepcions(0);
	}
	
	public Excepcions getExcepcions(int gravetatMinima){
		Excepcions errors = new Excepcions();
		for (Iterator<FormFacesExcepcio> it=excepcions.getItExcepcions(gravetatMinima); it.hasNext();){
			errors.nova(it.next());
		}
		for (Enumeration<Classe> cl = classes.elements(); cl.hasMoreElements();){
			Classe c = cl.nextElement();
			Excepcions ex = c.getExcepcions(gravetatMinima);
			for (Iterator<FormFacesExcepcio> it=ex.getItExcepcions(gravetatMinima); it.hasNext();){
				errors.nova(it.next());
			}
		}
		for (Enumeration<List<Classe>> lcl = llistes.elements(); lcl.hasMoreElements();){
			List<Classe> lc = lcl.nextElement();
			for (Iterator<Classe> it = lc.iterator(); it.hasNext();){
				Classe c = it.next();
				Excepcions ex = c.getExcepcions(gravetatMinima);
				for (Iterator<FormFacesExcepcio> itf=ex.getItExcepcions(gravetatMinima); itf.hasNext();){
					errors.nova(itf.next());
				}
			}
		}
		return errors;
	}
	
	public void novaExcepcio(FormFacesExcepcio e){
		excepcions.nova(e);
	}
	
	public void setClasse(String nom, Classe cl){
		if (classes.containsKey(nom)){
			if (classes.get(nom).getClass().getName().equalsIgnoreCase(cl.getClass().getName())){
				classes.put(nom, cl);
			} else {
				excepcions.nova(new FormFacesExcepcio("Classe "+nom+" no es del tipus correcte ", 4));
			}
		}
	}
	
	public Classe getClasse(String nom){
		if (classes.containsKey(nom)){
			return classes.get(nom);
		} else {
			return null;
		}
	}
	
	public void setLlista(String nom, List<Classe> lc){
		if (!lc.isEmpty()){
			if (llistes.containsKey(nom)){
				if (lc.get(0).getClass().getName().equalsIgnoreCase(classeLlista.get(nom))){
					llistes.put(nom, lc);
				} else {
					excepcions.nova(new FormFacesExcepcio("Llista "+nom+" no es del mateix tipus de classes: "+lc.get(0).getClass().getName()+" - "+classeLlista.get(nom), 4));
				}
			} else {
				excepcions.nova(new FormFacesExcepcio("Llista "+nom+" no existeix ", 4));
			}
		}
	}
	
	public List<Classe> getLlista(String nom){
		if (llistes.containsKey(nom)){
			return llistes.get(nom);
		} else {
			return null;
		}
	}
	
	public void setClasseList(String nom, Classe cl){
		if (llistes.containsKey(nom)){
			if (cl.getClass().getName().equalsIgnoreCase(classeLlista.get(nom))){
				llistes.get(nom).add(cl);
			} else {
				excepcions.nova(new FormFacesExcepcio("Llista "+nom+" no es del mateix tipus que classe", 4));
			}
		} else {
			excepcions.nova(new FormFacesExcepcio("Llista "+nom+" no existeix ", 4));
		}
	}
	
	public Enumeration<String> getClauClasses(){
		return classes.keys();
	}
	
	public Enumeration<String> getClauLlistes(){
		return llistes.keys();
	}
	
	public void clear(){
		excepcions.clear();
		for (Enumeration<String> par = getClauParametres(); par.hasMoreElements();){
			String param = par.nextElement();
			parametres.get(param).clear();
			
		}
		for (Enumeration<String> cla = getClauClasses(); cla.hasMoreElements();){
			String param = cla.nextElement();
			classes.get(param).clear();
		}
		for (Enumeration<String> lli = getClauLlistes(); lli.hasMoreElements();){
			String param = lli.nextElement();
			llistes.get(param).clear();
		}
	}

	public void clone(Classe c){
		if (c.getClass().getName().equalsIgnoreCase(this.getClass().getName())){
			for (Enumeration<String> par = getClauParametres(); par.hasMoreElements();){
				String param = par.nextElement();
				String valor = c.getValor(param);
				if (valor != null){
					setValor(param, valor);
				}
			}
			for (Enumeration<String> cla = getClauClasses(); cla.hasMoreElements();){
				String param = cla.nextElement();
				Classe valor = c.getClasse(param);
				if (valor != null){
					Classe actual = getClasse(param);
					actual.clone(valor);
				}
			}
			for (Enumeration<String> lli = getClauLlistes(); lli.hasMoreElements();){
				String param = lli.nextElement();
				List<Classe> valor = c.getLlista(param);
				if (valor != null){
					List<Classe> actual = this.getLlista(param);
					actual.clear();
					for (Iterator<Classe> it=valor.iterator(); it.hasNext();){
						actual.add(it.next());
					}
				}
			}
		}
	}
}
